<?php
class WP_auto_tag_setting {
	public function __construct() {
		add_action('admin_menu', array(&$this, 'add_menu'));
	}

	public function add_menu() {
		add_menu_page('WP Auto Tag', '自動標籤', 'manage_options', 'wp_auto_tag_setting', array(&$this, 'show_setting'));
		add_submenu_page('wp_auto_tag_setting', 'WP Auto Tag 設定', '設定', 'manage_options', 'wp_auto_tag_setting', array($this, 'show_setting'));
		add_submenu_page('wp_auto_tag_setting', 'WP Auto Tag 分享', '分享', 'manage_options', 'wp_auto_tag_share', array($this, 'show_share'));
		add_submenu_page('wp_auto_tag_setting', 'WP Auto Tag 測試', '測試', 'manage_options', 'wp_auto_tag_test', array($this, 'show_test'));
	}

	public function show_setting() {
		if( !empty($_POST['submit']) ) {
			$this->save_settings();
		}
		$analyitc_options = get_option('WP_auto_tag_analyitc_options');
		$options = get_option('WP_auto_tag_options');
		$this->show_header('WP Auto Tag 設定', 'widefat');
		$f_checked = checked($analyitc_options['ngram_type'], 'f', false);
		$r_checked = checked($analyitc_options['ngram_type'], 'r', false);
		$max_words = $analyitc_options['max_words'];
		$max_tags = $analyitc_options['max_tags'];
		$tag_max_len = 0;
		$tags = get_terms('post_tag');
		foreach( $tags as $key => $tag ) {
			if( !preg_match('/^[ 0-9a-z_\.\^-]*$/i', $tag->name) ) {
				$tag_max_len = max(mb_strlen($tag->name), $tag_max_len);
			}
		}
		$autotag_link_1 = checked($options['autotag_link'], '1', false);
		$autotag_link_0 = checked($options['autotag_link'], '0', false);
		$autotag_link_maxtimes = $options['autotag_link_maxtimes'];
		$autotag_link_home = checked($options['autotag_link_addin']['home'], true, false);
		$autotag_link_single = checked($options['autotag_link_addin']['single'], true, false);
		$autotag_link_page = checked($options['autotag_link_addin']['page'], true, false);
		$autotag_link_archive = checked($options['autotag_link_addin']['archive'], true, false);
		$autotag_link_search = checked($options['autotag_link_addin']['search'], true, false);
		echo <<<END
<thead><tr><th colspan="2">標籤分析</th></tr></thead>
<tr valign="top"><td>文字解析方向</td><td>
	<label><input type="radio" value="f" name="ngram_type" $f_checked /> 正向(由左到右) </label> &nbsp;
	<label><input type="radio" value="r" name="ngram_type" $r_checked /> 逆向(由右到左) </label><br />
	以我所知的研究報告來說，逆向的解析正確率會比較高。
</td></tr>
<tr valign="top"><td><label for="max_words">單一詞彙(標籤)最多字數</label></td><td>
	<input type="text" value="$max_words" name="max_words" id="max_words" size="3" /> 單字 (目前標籤的最長字數為 $tag_max_len)<br />本設定不會影響到英文單字的判斷。
</td></tr>
<tr valign="top"><td><label for="max_words">單次最多標籤數量</label></td><td>
	<input type="text" value="$max_tags" name="max_tags" id="max_tags" size="3" />
</td></tr>
<thead><tr><th colspan="2">通用設定</th></tr></thead>
<tr valign="top"><td rowspan="2">自動標籤連結</td><td>
	<label><input type="radio" value="1" name="autotag_link" $autotag_link_1 /> 啟用 </label> &nbsp;
	<label><input type="radio" value="0" name="autotag_link" $autotag_link_0 /> 停用 </label>
	<br />目前只針對文章有使用的標籤有效
</td></tr>
<tr><td>
	在 【 <input type="checkbox" value="home" name="autotag_link_addin[]" $autotag_link_home /> 首頁
	<input type="checkbox" value="single" name="autotag_link_addin[]" $autotag_link_single /> 文章
	<input type="checkbox" value="archive" name="autotag_link_addin[]" $autotag_link_archive /> 彙整
	<input type="checkbox" value="search" name="autotag_link_addin[]" $autotag_link_search /> 搜尋 】 的版面中添加連結<br />
	相同的標籤最多連結 <input type="text" value="$autotag_link_maxtimes" name="autotag_link_maxtimes" id="autotag_link_maxtimes" size="3" /> 次 ﹝0 代表不限制次數﹞<br />
</td></tr>
END;
		$this->show_footer('更新設定');
	}

	private function save_settings() {
		$analyitc_options = get_option('WP_auto_tag_analyitc_options');
		$analyitc_options['ngram_type'] = ($_POST['ngram_type'] == 'f') ? 'f' : 'r';
		$analyitc_options['max_words'] = intval($_POST['max_words']);
		$analyitc_options['max_tags'] = intval($_POST['max_tags']);
		update_option('WP_auto_tag_analyitc_options', $analyitc_options);
		$options = get_option('WP_auto_tag_options');
		$options['autotag_link'] = intval((bool)$_POST['autotag_link']);
		$options['autotag_link_maxtimes'] = intval($_POST['autotag_link_maxtimes']);
		$options['autotag_link_addin'] = array(
			'home' => false,
			'single' => false,
			'archive' => false,
			'search' => false
		);
		foreach( $_POST['autotag_link_addin'] as $value ) {
			switch($value) {
				case 'home':
					$options['autotag_link_addin']['home'] = true;
					break;
				case 'single':
					$options['autotag_link_addin']['single'] = true;
					break;
				case 'archive':
					$options['autotag_link_addin']['archive'] = true;
					break;
				case 'search':
					$options['autotag_link_addin']['search'] = true;
					break;
			}
		}
		update_option('WP_auto_tag_options', $options);
	}

	public function show_share() {
		if( !empty($_POST['submit']) ) {
			$this->do_share();
		}
		if( !empty($_POST['getditc']) ) {
			$this->do_getditc();
		}
		if( !empty($_POST['clear_history']) ) {
			$this->do_clear_history();
		}
		$options = get_option('WP_auto_tag_options');
		$this->show_header('WP Auto Tag 分享');
		echo <<<END
<tr><td colspan="2">
<p>某天的夜晚忽然想到自動替文章加入標籤的功能。可惜的是沒有找到可以良善運作於中文的自動標籤外。因而花了我一些時間做出這一個外掛。</p>
<p>中文的自動斷詞是一項高深的問題，甚至有博士生的論文就是專研於這一個領域。<br />主要的難題之一在於，中文的詞與詞中間沒有明確的分割，所以在一連串的單字當中要組合出符合作者想法的詞是問題的關鍵。</p>
<p>由於合法的詞庫取得不易，因此本外掛主要採用 <a href="http://en.wikipedia.org/wiki/N-gram" target="_blank">N gram</a> 的方法，並輔以網站現有的標籤進行分詞，最後再計算各詞權重計算取得結果。</p>
<p>但這樣只有獨樂樂啊！網路的世界就是要分享、要眾樂樂才會好玩才會精采的。<br />因此如果你願意的話，分享你的標籤資料給其他也有使用這一外掛的人，讓自動化的功能更加有效與精準！</p>
<p>分享你的標籤資料，並不會帶給額外的流量負擔。<br />會將你的標籤資料整理過後，上傳到我的網站，再讓其他人下載使用！</p>
</td></tr>
END;
		if( count($options['share_history']) > 0 ) {
			$date_format = get_option('links_updated_date_format');
			if( $options['last_get_time'] > 0 ) {
				$get_time = date($date_format, $options['last_get_time']);
			} else {
				$get_time = '尚未取得';
			}
			echo <<<END
<tr><td colspan="2"><input type="submit" name="getditc" class="button-primary" value="取得大家分享的字典檔" /><br />上次取得時間：$get_time<td></tr>
<tr><td colspan="2"><h3>分享紀錄</h3><table class="widefat"><thead><tr><th>分享時間</th><th>標籤數量</th><th>新詞彙數量</th></tr></thead>
END;
			$i = 0;
			foreach( $options['share_history'] as $share ) {
				$last_share_time = date($date_format, $share['time']);
				$total_word = $share['new_word'] + $share['old_word'];
				echo <<<END
<tr><td>$last_share_time</td><td>$total_word</td><td>$share[new_word]</td></tr>
END;
				$i++;
				if( $i > 10 ) {
					break;
				}
			}
			echo <<<END
<tfoot><tr><td colspan="3"><input type="submit" name="clear_history" class="button-primary" value="清除分享歷史紀錄" /></td></tr></tfoot></table></td></tr>
END;
		}
		$this->show_footer('分享我的標籤資料');
	}

	private function do_share() {
		global $user_identity;

		$options = get_option('WP_auto_tag_options');
		$upload = false;
		$tags = get_terms('post_tag');
		$dir = dirname(__FILE__);
		if( function_exists('gzopen') ) {
			$file = $dir . '/share_tag.data.gz';
			$fp = gzopen($file, 'w');
			gzwrite($fp, get_bloginfo('blogname') . "\n");
			gzwrite($fp, $user_identity . "\n");
			gzwrite($fp, get_bloginfo('wpurl') . "\n");
			foreach( $tags as $key => $tag ) {
				gzwrite($fp, sprintf('%s,%s,%s' . "\n", $tag->name, $tag->count, get_tag_link($tag)));
			}
			gzclose($fp);
			if( is_file($file) ) {
				$fields = array(
					'action' => 'uploadditc',
					'ditc' => '@' . $file
				);
				$ch = curl_init();
				curl_setopt($ch, CURLOPT_URL, 'http://localhost/wpautotag.com/share_ditc.php');
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
				curl_setopt($ch, CURLOPT_COOKIE, '');
				curl_setopt($ch, CURLOPT_TIMEOUT, 15);
				curl_setopt($ch, CURLOPT_POST, true);
				curl_setopt($ch, CURLOPT_POSTFIELDS, $fields);
				$body = @curl_exec($ch);
				$info = curl_getinfo($ch);
				@curl_close($ch);
				unlink($file);
				if( $info['http_code'] == 200 ) {
					$upload = true;
					$result = explode(',', $body);
					array_unshift($options['share_history'], array(
						'time' => current_time('timestamp'),
						'new_word' => intval($result[0]),
						'old_word' => intval($result[1])
					));
					update_option('WP_auto_tag_options', $options);
				}
			}
		}
		if( $upload ) {
			echo <<<END
<div id="message" class="updated"><p>
	<span style="color:green">分享成功！</span><br />感謝您的分享！讓所有使用者可以有更佳準確的分析結果。
</p></div>
END;
		} else {
			echo <<<END
<div id="message" class="error"><p>
	<span style="color:rad">分享失敗！</span><br />可能的原因有：使用到您伺服器不支援的技術；分享的檔案製作失敗：我的伺服器維修中。</span>
</p></div>
END;
		}
	}

	private function do_getditc() {
		$get = false;
		$options = get_option('WP_auto_tag_options');
		if ( function_exists('curl_init') && function_exists('gzinflate') ) {
			$fields = array(
				'action' => 'getditc',
			);
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, 'http://wpautotag.com/get_ditc.php');
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
			curl_setopt($ch, CURLOPT_COOKIE, '');
			curl_setopt($ch, CURLOPT_TIMEOUT, 15);
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $fields);
			$ditc = @curl_exec($ch);
			$info = @curl_getinfo($ch);
			@curl_close($ch);
			$ditc = gzinflate($ditc);
			if( file_put_contents(dirname(__FILE__) . '/include/share.ditc', $ditc) ) {
				$get = true;
				$options['last_get_time'] = current_time('timestamp');
				update_option('WP_auto_tag_options', $options);
			}
		}
		if( $get ) {
			echo <<<END
<div id="message" class="updated"><p>
	<span style="color:green">獲取成功！</span><br />成功的抓取最新的字典檔，自動分析的結果應該更加的準確了。
</p></div>
END;
		} else {
			echo <<<END
<div id="message" class="error"><p>
	<span style="color:rad">獲取失敗！</span><br />可能的原因有：使用到您伺服器不支援的技術；我的伺服器維修中。</span>
</p></div>
END;
		}
	}

	private function do_clear_history() {
		$options = get_option('WP_auto_tag_options');
		$history = array_pop($options['share_history']);
		$options['share_history'] = array();
		$options['share_history'][] = $history;
		update_option('WP_auto_tag_options', $options);
	}

	public function show_test() {
		$content = '';
		set_time_limit(8);
		if( !empty($_POST['submit']) ) {
			$content = $this->do_test();
		}
		$this->show_header('WP Auto Tag 測試');
		echo <<<END
<tr><th><label for="test_content">測試內容</label></th><td>
<textarea name="test_content" id="test_content" rows="10" cols="100">$content</textarea>
</td></tr>
END;
		$this->show_footer('分析生成文章的標籤');
	}

	private function do_test() {
		include(dirname(__FILE__) . '/include/analytics_class.php');
		$analyitc_options = get_option('WP_auto_tag_analyitc_options');
		$options = get_option('WP_auto_tag_options');
		$content = $_POST['test_content'];
		$analytics = new string_analytics();
		$analytics->SetOption($analyitc_options);
		if( $options['last_get_time'] > 0 ) {
			$ditc = dirname(__FILE__) . '/include/share.ditc';
			if( is_file($ditc) ) {
				$ditc = unserialize(file_get_contents($ditc));
				if( is_array($ditc) ) {
					$analytics->SetDict($ditc);
				}
			}
		}
		$tags = get_terms('post_tag', array('hide_empty' => false));
		foreach( $tags as $key => $tag ) {
			$tags[$tag->name] = $tag->count;
		}
		$analytics->SetDict($tags);
		$analytics->SetSource($content);
		$analytics->StartAnalysis();
		$autotags = $analytics->get_finally();
		$result = implode('<br />', $autotags);
		echo <<<END
<div id="message" class="updated"><p>
	<span style="color:green">測試分析完成！</span><br />分析的結果為：$result
</p></div>
END;
		return $content;
	}

	private function show_header($title, $class = 'form-table') {
		echo <<<END
<div class="wrap"><h2>$title</h2><form method="post" action=""><table class="$class">
END;
		return $content;
	}

	private function show_footer($value = '') {
		if( $value != '' ) {
			$value = '<input type="submit" name="submit" class="button-primary" value="' . $value . '" />';
		}
		$version = get_option('WP_auto_tag_version');
		echo <<<END
<tfoot><tr><td colspan="2">$value</td></tr></tfoot>
</table></form>
<p>目前外掛版本 $version</p>
</div>
END;
	}
}

new WP_auto_tag_setting();
?>