<?php
/*
Plugin Name: WP auto tag
Plugin URI: http://wpteach.com/wp-auto-tag
Description: Automatically add relevant tags to new posts. 自動化的幫文章添加標籤。
Version: 0.0.5.1
Author: Richer Yang
Author URI: http://fantasyworld.idv.tw/
*/

register_activation_hook(__FILE__, 'wp_auto_tag_init');
function wp_auto_tag_init() {
	$test = get_option('WP_auto_tag_analyitc_options', array());
	if( count($test) == 0 ) {
		$options = array(
			'share_history' => array(),
			'last_get_time' => 0,
			'autotag_link' => 1,
			'autotag_link_maxtimes' => 3,
			'autotag_link_addin' => array(
				'home' => true,
				'single' => true,
				'archive' => true,
				'search' => false
			)
		);
		add_option('WP_auto_tag_options', $options, 'WP auto tag Options');
		$analyitc_options = array(
			'ngram_type' => 'r',
			'max_words' => '6',
			'max_tags' => '10'
		);
		add_option('WP_auto_tag_analyitc_options', $analyitc_options, 'WP auto tag analyitc Options');
		add_option('WP_auto_tag_version', '0.0.4', 'WP auto tag version');
	}
}

class WP_auto_tag {
	private $options;

	public function __construct() {
		$this->options = get_option('WP_auto_tag_options');
		if( is_admin() ) {
			include(dirname(__FILE__) . '/include/update.php');
			include(dirname(__FILE__) . '/admin.php');
			add_action('add_meta_boxes_post', array(&$this, 'add_meta_box'));
			add_action('wp_ajax_wp-auto-tag', array(&$this, 'make_tags'));
			add_filter('plugin_action_links', array(&$this, 'add_setting_links'), 10, 2);
		} else {
			if( $this->options['autotag_link'] ) {
				add_filter('the_content', array(&$this, 'auto_taglink'));
			}
		}
	}

	public function add_meta_box() {
		add_action('admin_head', array(&$this, 'add_scripts'));
		add_meta_box('tagsdiv-wpautotag', '自動標籤', array(&$this, 'show_meta_box'), 'post', 'side', 'default');
	}

	public function show_meta_box($post) {
		echo <<<END
<div class="inside">
<p><input type="button" class="button doautotag" value="辨識標籤"></p>
</div>
END;
	}

	public function add_scripts() {
		echo <<<END
<script type="text/javascript">
jQuery(document).ready(function($) {
	$('.doautotag').click(function(){
		var _item = $('.doautotag');
		var _content, _value;
		_content = tinymce.DOM.get('content').value;
		_value = _item.attr('value');
		_item.attr('disabled', true).attr('value', 'analyzing! Please wait');
		jQuery.post(ajaxurl, {
			action: 'wp-auto-tag',
			content: _content,
			title: $('#title').attr('value')
		}, function(data){
			$('#new-tag-post_tag').attr('value', data);
			_item.attr('disabled', false).attr('value', _value);
		});
	});
});
</script>
END;
	}

	public function make_tags() {
		include(dirname(__FILE__) . '/include/analytics_class.php');
		$analyitc_options = get_option('WP_auto_tag_analyitc_options');
		$content = $_POST['title'] . ' ' . $_POST['title'] . ' ' . $_POST['content'];
		$analytics = new string_analytics();
		$analytics->SetOption($analyitc_options);
		if( $this->options['last_get_time'] > 0 ) {
			$ditc = dirname(__FILE__) . '/include/share.ditc';
			if( is_file($ditc) ) {
				$ditc = unserialize(file_get_contents($ditc));
				if( is_array($ditc) ) {
					$analytics->SetDict($ditc);
				}
			}
		}
		$tags = get_terms('post_tag', array('hide_empty' => false));
		foreach( $tags as $key => $tag ) {
			$tags[$tag->name] = $tag->count;
		}
		$analytics->SetDict($tags);
		$analytics->SetSource($content);
		$analytics->StartAnalysis();
		$autotags = $analytics->get_finally();
		echo(implode(',', $autotags));
		exit();
	}

	public function add_setting_links($links, $file) {
		if( $file == plugin_basename(__FILE__) ) {
			$links[] = '<a href="admin.php?page=wp_auto_tag_setting">設定</a>';
		}
		return $links;
	}

	public function auto_taglink($content) {
		$out = true;
		if( is_home() && $this->options['autotag_link_addin']['home'] ) {
			$out = false;
		}
		if( is_single() && $this->options['autotag_link_addin']['single'] ) {
			$out = false;
		}
		if( is_archive() && $this->options['autotag_link_addin']['archive'] ) {
			$out = false;
		}
		if( is_search() && $this->options['autotag_link_addin']['search'] ) {
			$out = false;
		}
		if( $out ) {
			return $content;
		}

		global $post;
		$no_replace = '&&&';
		while( strpos($content, $no_replace) !== false ) {
			$no_replace .= '&';
		}
		$search = array();
		$replace = array();
		$tags = get_the_terms($post->ID, 'post_tag');
		foreach( $tags as $tag ) {
			$replace = '<a class="autotag-link" href="' . esc_url(get_term_link($tag, 'post_tag')) . '" rel="tag">' . $tag->name . '</a>';
			$content = preg_replace('@(<a[^>]*>[^<]*)' . preg_quote($tag->name) . '([^<]*<\/a[^>]*>)@si', '$1' . $no_replace . '$2', $content);
			$content = preg_replace('@(<\/?[^>]*)' . preg_quote($tag->name) . '([^>]*>)@si', '$1' . $no_replace . '$2', $content);
			$content = $this->mb_str_replace($tag->name, $replace, $content, $this->options['autotag_link_maxtimes']);
			$content = $this->mb_str_replace($no_replace, $tag->name, $content);
		}
		return $content;
	}

	private function mb_str_replace($search, $replace, $subject, $count = 0) {
		$search_length = mb_strlen($search);
		$replace_length = mb_strlen($replace);
		$offset = 0;
		if( $count == 0 ) {
			while( ($start = mb_strpos($subject, $search, $offset)) !== false ) {
				$subject = mb_substr($subject, 0, $start) . $replace . mb_substr($subject, $start + $search_length);
				$offset = $start + $replace_length;
			}
		} else {
			for( $j = $count; $j > 0; $j-- ) {
				$start = mb_strpos($subject, $search, $offset);
				if( $start !== false ) {
					$subject = mb_substr($subject, 0, $start) . $replace . mb_substr($subject, $start + $search_length);
					$offset = $start + $replace_length;
				} else {
					break;
				}
			}
		}
		return $subject;
	}
}

new WP_auto_tag();
?>